/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

/**
 * Base class for {@link Phenotype}s which consists of {@link Phenotype}s.
 * 
 * @author diewald
 */
public class PhenotypeBase implements Phenotype {

	/**
	 * Map which correlates the {@link Phenotype} classes with their corresponding {@link Phenotype}
	 * s, out of which this {@link Phenotype} consists.
	 */
	protected Map<Class<? extends Phenotype>, Phenotype> subPhenotypes = new HashMap<>();

	/** Flag to mark a phenotype as being processed by sub-Decoders. */
	private boolean isSubDecoding;

	/** {@inheritDoc} */
	@Override
	public <CP extends Phenotype> CP getSubPhenotype(Class<CP> subGenotypeClass) {
		return (CP)subPhenotypes.get(subGenotypeClass);
	}

	/** {@inheritDoc} */
	@Override
	public <CP extends Phenotype> void putSubPhenotype(Class<CP> subGenotypeClass, CP subPhenotype) {
		subPhenotypes.put(subGenotypeClass, subPhenotype);
	}

	/** {@inheritDoc} */
	@Override
	public List<Phenotype> getSubPhenotypes() {
		ArrayList<Phenotype> retList = new ArrayList<>();
		subPhenotypes.values().stream().forEachOrdered(p -> retList.add(p));
		return retList;
	}

	/** {@inheritDoc} */
	@Override
	public Iterator<Entry<Class<? extends Phenotype>, Phenotype>> iterator() {
		return subPhenotypes.entrySet().iterator();
	}

	/** {@inheritDoc} */
	@Override
	public Set<Class<? extends Phenotype>> keySet() {
		return subPhenotypes.keySet();
	}

	/** {@inheritDoc} */
	@Override
	public Class<? extends Phenotype> getPTypeToRegister() {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public void setSubDecoding(boolean isSubDecoding) {
		this.isSubDecoding = isSubDecoding;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isSubDecoding() {
		return isSubDecoding;
	}
}
