/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.selection;

import org.opt4j.core.optimizer.Operator;

/**
 * Selection strategy that selects operators in a round-robin fashion (in the registration order of
 * the operators).
 * 
 * @author barner
 */
public class RoundRobinOperatorSelector extends OperatorSelectorBase<Operator<?>> {
	/** Operator selected for the current round */
	private int currentOperator;

	/** {@inheritDoc} */
	@Override
	protected boolean evaluateInternal(Operator<?> operator) {
		boolean rval = (operatorToIndexMap.get(new OperatorKey<>(operator)) == currentOperator);
		return rval;
	}

	/** {@inheritDoc} */
	@Override
	protected void finalizeSelectionRoundInternal() {
		currentOperator++;
		currentOperator %= operatorToIndexMap.size();
	}
}
